__doc__= """
TALAN
=====
A Technical Algorithmic Analytics Python Package
"""
__version__ = "0.1.4"
__author__ = 'Ted Hong'
__credits__ = 'Beyondbond Risk Lab'
__name__ = 'talan'
from .talan_wrap import *
__all__= list(talan_vars.keys())
